// StarPlot - A program for interactively viewing 3D maps of stellar positions.
// Copyright (C) 2000  Kevin B. McCarty
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


/*
  filedialogs.cc
  This file contains code for the dialog boxes in the StarPlot File menu.
*/

#include <gtk/gtk.h>
#include "starplot.h"

/* File selection and merge dialogue */

/* Forward declaration of the file selection graphical interface */
void file_open_or_merge
(void (*select_function)(GtkWidget *, GtkFileSelection *));

/* This opens a new file */
void file_ok_sel( GtkWidget        *w,
		  GtkFileSelection *fs )
{
  starrules.ChartFileNames = StringList((char *)gtk_file_selection_get_filename
					(GTK_FILE_SELECTION (fs)));
  redraw_all(FILE_CHANGE);
}

/* This merges the selected file with the other(s) already open */
void merge_ok_sel( GtkWidget        *w,
		   GtkFileSelection *fs )
{
  const char *filename = 
    (char *)gtk_file_selection_get_filename(GTK_FILE_SELECTION (fs));
  				
  // make sure the file we want to merge isn't already open
  //  (not foolproof due to the possibility of symlinks, etc., but better
  //  than nothing)
  for (unsigned int i = 0; i < starrules.ChartFileNames.size(); i++)
    if (strcmp(starrules.ChartFileNames[i], filename) == 0) return;

  starrules.ChartFileNames.append(filename);
  redraw_all(FILE_CHANGE);
}

/* We need two functions for the menu generator, but they both call
 *  the same interface, just choose the correct file selection function */
void file_open()   { file_open_or_merge (file_ok_sel);  }
void file_merge()  { file_open_or_merge (merge_ok_sel); }

/* The interface which creates the dialog */
void file_open_or_merge 
(void (*select_function)(GtkWidget *, GtkFileSelection *))
{ 
  GtkWidget *filew;

  if (select_function == file_ok_sel)
    filew = gtk_file_selection_new ("StarPlot: Select data file to open");
  else
    filew = gtk_file_selection_new ("StarPlot: Select data file to merge");
  gtk_window_set_modal (GTK_WINDOW (filew), TRUE);
  gtk_signal_connect (GTK_OBJECT (filew), "destroy",
		      GTK_SIGNAL_FUNC (gtk_widget_destroy), 
		      GTK_WIDGET (filew));

  /* Connect the ok_button to the appropriate file selection function */
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (filew)->ok_button),
		      "clicked", GTK_SIGNAL_FUNC (select_function), filew);
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (filew)->ok_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (filew));

  /* Connect the cancel_button to destroy the widget */
  gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION
					 (filew)->cancel_button),
			     "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			     GTK_OBJECT (filew));

  /* Set the default filename */
  gtk_file_selection_set_filename (GTK_FILE_SELECTION(filew), 
                                   DATADIR "/sample.stars");
  gtk_widget_show(filew);
}


/* These are not yet implemented: */

void params_open() { }
void file_save() { }
void file_params() { }
